% Dense numerical model or a vibrating string

string_params;

nx = 100;
dx = L/(nx+1);  % two extremal fixed points are not included in n
x = (1:nx)*dx;

D2 = laplace_dis(nx, dx, "dense");

%% time domain simulation
dt = 1e-4; %s 
dt = 1e-5; %s 
dt_max = 2/(nx*2*pi*f1*0.64); % nb: pi*0.64 ~= 2. This fact may be proven by spectral analysis of D2 ?
assert(dt<dt_max, sprintf('dt too large for symplectic Euler integrator. Use dt < %.2g s', dt_max))

nt = 5000 + 1;% pts
t = (0:(nt-1))*dt;
fprintf('t fin = %.1f ms\n', t(end)*1e3)

% initial condition
n_fing = floor(nx/3);
ymax = 1e-2 ; % m
y = ymax * [linspace(0,1,n_fing) linspace(1,0,nx-n_fing)]';
% y = ymax * [linspace(0,1,n_fing+1) linspace(1,0,nx-n_fing+1)]';
% y = y(2:end-1);
v = zeros(nx,1);

% observations along time:
y_mid = zeros(nt,1); % at nx/2
y_third = zeros(nt,1); % at nx/3
y_fourth = zeros(nt,1); % at nx/4

% record string animation
vid_rec = 0;
if vid_rec
    fname = sprintf('string_n%d.avi', nx);
    vid = VideoWriter(fname);
    open(vid);
    fig2 = figure(2);
end

tic
for k=1:nt
    y_mid(k)    = y(floor(nx/2));
    y_third(k)  = y(floor(nx/3));
    y_fourth(k) = y(floor(nx/4));
    
    % Symplectic Euler integration:
    acc = (T/mu)*D2*y - (a/mu)*v;
    v_next = v + acc*dt;
    y_next = y + v_next*dt;
    
    y = y_next;
    v = v_next;
    
    if vid_rec
        % Plot and save animation:
        plot([0 L], [0 0], 'ko') % fixed extremal points
        hold on
        plot(x, y*1e3, '-o', 'MarkerSize', 2)
        grid on

        title(sprintf('100 Hz string with %d pts at t=%.1f ms', nx, t(k)*1e3))
        ylim([-ymax, ymax]*1.2*1e3)
        xlabel('position (m)')
        ylabel('y (mm)')
        writeVideo(vid, getframe(fig2));
        hold off
    end %if vid_rec
end % for k

if vid_rec
    close(vid);
end %if vid_rec

toc % 100 ms for nx=100, nt=5001

% Time plot
figure(1)
hold on

plot(t*1000, y_mid*1e3)
plot(t*1000, y_third*1e3)
plot(t*1000, y_fourth*1e3)

grid on
legend('y(nx/2)', 'y(nx/3)', 'y(nx/4)')
xlabel('time (ms)')
ylabel('y (mm)')
title(sprintf('100 Hz string with %d pts. dt=%.0f s', nx, dt*1e6))


% String plot at end of simulation
figure(2)
plot([0 L], [0 0], 'ko') % fixed extremal points
hold on
plot(x, y*1e3, '-o', 'MarkerSize', 2)
grid on

title(sprintf('100 Hz string with %d pts at t=%.1f ms', nx, t(k)*1e3))
ylim([-ymax, ymax]*1.2*1e3)
xlabel('position (m)')
ylabel('y (mm)')

%% Bode diagram

f_list = linspace(0, 1000, 2000);
nf = length(f_list);

g_1 = zeros(nf,1);
g_mid = zeros(nf,1);
g_third = zeros(nf,1);
g_fourth = zeros(nf,1);
a = 0.1;

u = ones(nx,1);
% u = zeros(nx,1)
% u(1) = 1

tic
for k=1:nf
    w = 2*pi*f_list(k);
    M = (-w^2 + 1j*w*a/mu)*eye(nx) - T/mu*D2;
    g = abs(M \ u);
    g_1(k) = g(1);
    g_mid(k) = g(floor(nx/2));
    g_third(k) = g(floor(nx/3));
    g_fourth(k) = g(floor(nx/4));
end % for each freq
toc % 0.8 s for nx=200, nf=2000

figure(3)
semilogy(f_list, g_mid, '-')
hold on
semilogy(f_list, g_third, '-')
semilogy(f_list, g_fourth, '-')
semilogy(f_list, g_1, '-')
xlabel('f excitation (Hz)')
legend('g(nx/2)', 'g(nx/3)', 'g(nx/4)', 'g(1)')
grid
title(sprintf('Bode of a 100 Hz string with %d pts, uniform acceleration', nx))

%% Mode analysis

tic
[V,D] = eig(T/mu*D2);
toc % 50 ms for n = 1000, 38s for n=10e3
D = diag(D);
fn = flip(sqrt(-D)/(2*pi));
V = fliplr(V);

figure(4)
plot(fn, '+:')
hold on
plot((1:nx)*fn(1), '--')
xlabel('mode number')
ylabel('fn (Hz)')
title(sprintf('Modes of a 100 Hz string with %d pts', nx))

grid

figure(5)
plot([0 L], [0 0], 'ko') % fixed extremal points
hold on
plot(x, V(:,1:3), '-o', 'MarkerSize', 2)
plot(x, V(:,end), '-x', 'MarkerSize', 4)

grid on
legend('ext', '1', '2', '3', sprintf('%d', nx))
xlabel('position (m)')
ylabel('y (m)')
title(sprintf('Modes of a 100 Hz string with %d pts', nx))

% max pulsation:
% (should be smaller than 2/dt for the stability of symplectic Euler integrator)
fprintf('w(%d) = %.0f Hz = %.2f krad/s\n', nx, fn(end), fn(end)*2*pi/1e3)