% Sparse numerical model or a vibrating string

string_params;

nx = 1000;
dx = L/(nx+1);  % two extremal fixed points are not included in n
x = (1:nx)*dx;

% Sparse discrete Laplacian d./dx2 operator
D2s = laplace_dis(nx, dx, "sparse");

% check result:
%disp(full(D2s) * dx^2)

% density:
nnz(D2s) / numel(D2s)

% display sparsity pattern
%spy(D2s)

% A quick look at the inverse of D2, which is dense!
% It corresponds to the string positions such that f= [0..0 1 0..0]
%imagesc(inv(D2s))

%% time domain simulation
dt = 1e-6; %s 
dt_max = 2/(nx*2*pi*f1*0.64); % nb: pi*0.64 ~= 2. This fact may be proven by spectral analysis of D2 ?
assert(dt<dt_max, sprintf('dt too large for symplectic Euler integrator. Use dt < %.2g s', dt_max))

nt = 10000 + 1;% pts
t = (0:(nt-1))*dt;
fprintf('t fin = %.1f ms\n', t(end)*1e3)

% initial condition
n_fing = floor(nx/3);
ymax = 1e-2 ; % m
y = ymax * [linspace(0,1,n_fing) linspace(1,0,nx-n_fing)]';
v = zeros(nx,1);

% observations along time:
y_mid = zeros(nt,1); % at nx/2
y_third = zeros(nt,1); % at nx/3
y_fourth = zeros(nt,1); % at nx/4

tic
for k=1:nt
    y_mid(k)    = y(floor(nx/2));
    y_third(k)  = y(floor(nx/3));
    y_fourth(k) = y(floor(nx/4));
    
    % Symplectic Euler integration:
    acc = (T/mu)*D2s*y - (a/mu)*v;
    v_next = v + acc*dt;
    y_next = y + v_next*dt;
    
    y = y_next;
    v = v_next;

end % for k

toc % 36 ms for nx=100, 150 ms for nx=1000 (nt=5001)

% Time plot
figure(1)
hold on

plot(t*1000, y_mid*1e3)
plot(t*1000, y_third*1e3)
plot(t*1000, y_fourth*1e3)

grid on
legend('y(nx/2)', 'y(nx/3)', 'y(nx/4)')
xlabel('time (ms)')
ylabel('y (mm)')
title(sprintf('100 Hz string with %d pts. dt=%.0f s', nx, dt*1e6))


% String plot at end of simulation
figure(2)
plot([0 L], [0 0], 'ko') % fixed extremal points
hold on
plot(x, y*1e3, '-o', 'MarkerSize', 2)
grid on

title(sprintf('100 Hz string with %d pts at t=%.1f ms', nx, t(k)*1e3))
ylim([-ymax, ymax]*1.2*1e3)
xlabel('position (m)')
ylabel('y (mm)')

%% Bode diagram

% to be done

%% Mode analysis

n_modes = 10;

tic
% [V,D] = eig(T/mu*D2s); NOT ALLOWED
[V,D] = eigs(T/mu*D2s, n_modes, 'smallestabs');
D = diag(D);
fn = sqrt(-D)/(2*pi);
toc % 7 ms for n = 1000, 60 ms for n=10e3

figure(4)
plot(fn, '+:')
hold on
plot((1:n_modes)*fn(1), '--')

grid on
xlabel('mode number')
ylabel('fn (Hz)')
title(sprintf('%d first modes of a 100 Hz string with %d pts', n_modes, nx))


figure(5)
plot([0 L], [0 0], 'ko') % fixed extremal points
hold on
plot(x, V(:,1:3), '-o', 'MarkerSize', 2)
plot(x, V(:,end), '-x', 'MarkerSize', 4)

grid on
legend('ext', '1', '2', '3', sprintf('%d', n_modes))
xlabel('position (m)')
ylabel('y (m)')
title(sprintf('Modes of a 100 Hz string with %d pts', nx))